/****************************************************************************
** Meta object code from reading C++ file 'kxmlguifactory.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.17)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../kxmlgui-5.116.0/src/kxmlguifactory.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kxmlguifactory.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.17. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KXMLGUIFactory_t {
    QByteArrayData data[14];
    char stringdata0[205];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KXMLGUIFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KXMLGUIFactory_t qt_meta_stringdata_KXMLGUIFactory = {
    {
QT_MOC_LITERAL(0, 0, 14), // "KXMLGUIFactory"
QT_MOC_LITERAL(1, 15, 11), // "clientAdded"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 14), // "KXMLGUIClient*"
QT_MOC_LITERAL(4, 43, 6), // "client"
QT_MOC_LITERAL(5, 50, 13), // "clientRemoved"
QT_MOC_LITERAL(6, 64, 13), // "makingChanges"
QT_MOC_LITERAL(7, 78, 14), // "shortcutsSaved"
QT_MOC_LITERAL(8, 93, 18), // "configureShortcuts"
QT_MOC_LITERAL(9, 112, 21), // "bAllowLetterShortcuts"
QT_MOC_LITERAL(10, 134, 13), // "bSaveSettings"
QT_MOC_LITERAL(11, 148, 28), // "showConfigureShortcutsDialog"
QT_MOC_LITERAL(12, 177, 20), // "changeShortcutScheme"
QT_MOC_LITERAL(13, 198, 6) // "scheme"

    },
    "KXMLGUIFactory\0clientAdded\0\0KXMLGUIClient*\0"
    "client\0clientRemoved\0makingChanges\0"
    "shortcutsSaved\0configureShortcuts\0"
    "bAllowLetterShortcuts\0bSaveSettings\0"
    "showConfigureShortcutsDialog\0"
    "changeShortcutScheme\0scheme"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KXMLGUIFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       5,    1,   62,    2, 0x06 /* Public */,
       6,    1,   65,    2, 0x06 /* Public */,
       7,    0,   68,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    2,   69,    2, 0x0a /* Public */,
       8,    1,   74,    2, 0x2a /* Public | MethodCloned */,
       8,    0,   77,    2, 0x2a /* Public | MethodCloned */,
      11,    0,   78,    2, 0x0a /* Public */,
      12,    1,   79,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Int, QMetaType::Bool, QMetaType::Bool,    9,   10,
    QMetaType::Int, QMetaType::Bool,    9,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,

       0        // eod
};

void KXMLGUIFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KXMLGUIFactory *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clientAdded((*reinterpret_cast< KXMLGUIClient*(*)>(_a[1]))); break;
        case 1: _t->clientRemoved((*reinterpret_cast< KXMLGUIClient*(*)>(_a[1]))); break;
        case 2: _t->makingChanges((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->shortcutsSaved(); break;
        case 4: { int _r = _t->configureShortcuts((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->configureShortcuts((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->configureShortcuts();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->showConfigureShortcutsDialog(); break;
        case 8: _t->changeShortcutScheme((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (KXMLGUIFactory::*)(KXMLGUIClient * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KXMLGUIFactory::clientAdded)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (KXMLGUIFactory::*)(KXMLGUIClient * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KXMLGUIFactory::clientRemoved)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (KXMLGUIFactory::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KXMLGUIFactory::makingChanges)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (KXMLGUIFactory::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KXMLGUIFactory::shortcutsSaved)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KXMLGUIFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KXMLGUIFactory.data,
    qt_meta_data_KXMLGUIFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KXMLGUIFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KXMLGUIFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KXMLGUIFactory.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KXMLGUIFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KXMLGUIFactory::clientAdded(KXMLGUIClient * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KXMLGUIFactory::clientRemoved(KXMLGUIClient * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void KXMLGUIFactory::makingChanges(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void KXMLGUIFactory::shortcutsSaved()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
